/************************************************************************/
/*									*/
/*						 XL DATASOURCE, Inc.	*/
/*  Copyright (C) 1988, 1989, 1990, 1991      PO Box 28-B   Station 16	*/
/*						 Lakewood, CO  80228	*/
/*									*/
/************************************************************************/

#define BITMAP_MAX 500000L  /* largest number of elements allowed */

/* these macros provide for very fast bit mapping 
   (with 'n' a variable, greater than 65000 bits manipulated per second
   in a for loop -- GET_BIT is nearly twice THAT fast because no assignment
   is performed...
   if 'n' is a constant the compiler does most of the work and the speeds
   double...)

   SET_BIT:    sets bit n to 1
   RESET_BIT:  sets bit n to 0
   TOGGLE_BIT: flips bit n
   GET_BIT:    is equal to the value of bit n -- this may be one of
	       0, 1, 2, 4, 8, 16, 32, 64, 128 -- it is easy to see if the
	       requested bit is set (non-zero) or not (0)
*/

#define SET_BIT(bmap, n)    (*(bmap + (n >> 3)) |= (1 << (n & 0x00000007L)))
#define RESET_BIT(bmap, n)  (*(bmap + (n >> 3)) &= ~(1 << (n & 0x00000007L)))
#define TOGGLE_BIT(bmap, n) (*(bmap + (n >> 3)) ^= (1 << (n & 0x00000007L)))
#define GET_BIT(bmap, n)    (*(bmap + (n >> 3)) & (1 << (n & 0x00000007L)))

/* a bitmap is just a BYTE buffer */
typedef unsigned char *BITMAP;

BITMAP create_bitmap(unsigned long num_elements);

/*=========================================================================*/
