/*
**	lcpxm.h - xmodem engine for litecomm - header file
**			   Copyright (c) 1991 - Information Technology, Ltd.
**			   All Rights Reserved
**	REVISIONS
**		01		05/20/91		modified to support multiports
*/

/*
**	ASCII and XMODEM control characters
*/

#define SOH 1					/* start of header */
#define STX 2					/* start of text - YModem variant */
#define EOT 4					/* end of transmission */
#define ACK 6					/* true acknowledge */
#define NAK 0x15				/* false acknowledge */
#define DLE 0x10				/* DLE char for WXmodem */
#define SYN 0x16				/* SYN char for WXmodem */
#define CRC 'C'					/* request CRC mode */
#define CAN 0x18				/* cancel transmission */

/*
**	User accessable system equates
*/

#define RETRY		 25					/* no. of retrys before abort */
#define RECSIZE		 128				/* transfer record size */
#define YMRECSIZE	1024				/* transfer record size */
#define NORMAL		   1				/* normal xmodem timeouts */
#define RELAXED		   3				/* relaxed (longer) timeouts */

/*
** additional constants used for status codes from x,ymodem
** routines
*/
#define SUCCESS		('S')
#define RETRIES		('R')
#define TOUT		('T')
#define RESEND		('E')
#define DUPSEQ		('D')
#define ABORTED		('A')
#define FATAL		('F')
#define FATALIO		('I')

#define WAITSOH		10					/* Waiting for SOH */
#define WAITBLK		11					/* Waiting for Block */
#define WAITHDSK	12					/* Waiting for Handshake */
#define WAITACK		13					/* Waiting for Acknowledgement */
#define WAITFNB		14					/* Waiting for File Name Block */
#define XDISKERR	15					/* Disk read/write Error */
#define XGOTFILE	18					/* Got File Name Block */
#define XBADFILE	19					/* Can't find file */
#define XRECBLK		20					/* report of block size */

/*
** function prototypes
*/

#ifdef __cplusplus
extern "C" {
#endif

void updcrc(PROTO *, unsigned char);
int lcxteot(PROTO *);
int lcpxmrec(PROTO *);
int lcpxmsnd(PROTO *p);
int lcpwxrec(PROTO *p);
int lcpwxsnd(PROTO *p, int *nrec);
void lcym_send(PROTO *, char **);
void lcym_recv(PROTO *, char *);
void lcxm_send(PROTO *, char *);
void lcxm_recv(PROTO *, char *);
int lwxtrec(unsigned, unsigned char *, int *);
int lwxrrec(unsigned, unsigned char *);

#ifdef __cplusplus
};
#endif
