/*
**  LiteComm Demo Program - ZModem Receive
**  Copyright 1992 - Information Technology, Ltd.
**					 All Rights Reserved
*/
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <conio.h>

#include "litecomm.h"
#include "lcproto.h"
#include "lczm.h"

#ifdef M_I86
/*
** these definitions satisfy externs in lcbbs.c - 
** not actually used but called in by MSC linkage
*/
char MODEMSET0[] = "ATZ\r";
char MODEMSET1[] = "ATT E0\r";
char MODEMSET2[] = "ATC1 V0 X1 S0=1 M1\r";
#endif

/*
** Required by getopt
*/
static char optlist[] = "b:B:p:P:f:F:hH";		// list of valid options
extern char *optarg;
extern int	optind;
extern int	opterr;
int getopt(int argc, char *argv[], char *optstring);

/*
** option variables
*/
static unsigned port = 1;
static long baudrate = 2400L;
static char flow = 'n';

/*
** function prototypes
*/
void usage(void);
void fail(int);
void lcz_status(int, long, void *);
int lcz_abort(void);

void main(int argc, char *argv[])
{
	int		i;
	PROTO	*pcb;

	optind = 0;
	opterr = 1;							// function generated messages
	puts("\nLCZR - Copyright 1992, Information Technology, Ltd.");
	while ((i = getopt(argc, argv, optlist)) != EOF)
		switch (i)
		{
			case 'p':
			case 'P':
				port = atoi(optarg);
				if ((port < 1) || (port > 4))
					usage();
				break;
			case 'b':
			case 'B':
				baudrate = atol(optarg);
				if (baudrate < 300)
					usage();
				break;
			case 'f':
			case 'F':
				switch (*optarg)				// examine first char
				{
					case 's':
					case 'S':
					case 'h':
					case 'H':
					case 'n':
					case 'N':
						flow = tolower(*optarg);
						break;
					default:
						usage();
						break;
				}
				break;
			case '?':
			default:
				puts("Invalid Option");
			case 'h':
			case 'H':
				usage();
				break;
		}

/*
** announce what we're trying to do
*/
	printf("LCZR active on COM%d, line speed = %ld\n", port, baudrate);

/*
** attempt to open the port -
**	deliberately use a large input buffer to support higher transmission
**  speeds with ZModem
*/
	if (lc_open(port, baudrate, NPARITY, BIT8, STOP1, 24000, 128, TRUE) == ERR)
		fail (_lc_error);

/*
** if flow control has been requested, turn it on now
*/
	if (flow == 'h')							// hardware flow control
		lc_sethwflow(port, (USERTS|USECTS));
	else
		if (flow == 's')						// software flow control
			lc_xoff(port, TRUE);

/*
** if this port is a 16550, activate the fifo
*/
	if (lc_get_type(port) == P16550A)
	{
		puts("\nFifo buffer Activated");
		lc_fifo_on(port);
	}
/*
** allocate a pcb and prepare for ZModem transfer
*/
	if ((pcb = lcp_alloc(port)) == NULL)
		fail(_lc_error);
	lcp_setproto(pcb, ZMODEM, 0);

/*
** set up the pointers for the status, abort functions
*/
	lcp_status = lcz_status;
	lcp_abort = lcz_abort;

/*
** do the transfer into the current working directory
*/
	lczm_recv(pcb, baudrate, ".\\");

/*
** bring to a normal conclusion
*/
	if (lc_get_type(port) == P16550A)
	{
		puts("\nFifo buffer Deactivated");
		lc_fifo_off(port);
	}
	lc_close(port, TRUE);
}

void lcz_status(int mcode, long intval, void *ptr)
{
	switch (mcode)
	{
		case ZBADFILE:
			printf("Unable to open file %ld\n", intval);
			break;
		case ZBADHDR:
			puts("Bad header received");
			break;
		case ZBADPOS:
			puts("Positioning Error");
			break;
		case ZDISKERR:
			printf("Disk error occurred at %ld", intval);
			break;
		case ZFCREAT:
			puts("Error trying to create file");
			break;
		case ZFOPEN:
			puts("Error trying to open file");
			break;
		case ZINITTO:
			puts("Initialization Timeout");
			break;
		case ZGOTFILE:
			printf("Receiving file %16.16s\n",((TELINK *)ptr)->filename);
			break;
		case ZNOCAR:
			puts("Loss of Carrier Detected");
			break;
		case ZPKTLEN:
			printf("Packet of length %ld received", intval);
			break;
		case ZRECOVER:
			puts("Attempting to restart transfer");
			break;
		case ZSEEKERR:
			puts("Seek Error");
			break;
		case ZTIMER:
			puts("TIMEOUT");
			break;
		case ZXFRCAN:
			puts("Transfer Cancelled");
			break;
		case ZRECBLK:
			printf("Received %ld\r", intval);
			break;
		default:
			printf("Unknown Message Code: %d\n", mcode);
			break;
	}
}

int lcz_abort(void)
{
	int	ch;

	if (kbhit())
	{
		if ((ch = getch()) == 0) 				  // function key ?
		{
			ch = getch();
			return 0;
		}
		if (ch == 0x1b)							  // ESC pressed
			return 1;
	}
	return 0;
}

void fail(int mcode)
{
	printf("\nFailed with _lc_error = %d", mcode);
	exit(4);
}

void usage(void)
{
	puts("\nUsage:");
	puts("\tlczr [-b rate] [-p 1|2|3|4] [-f h|s|n]");
	exit(2);
}
