#define MAIN

#include "xllist.h"

#define ABORT 0			/* for c_break () */

/***** prototypes *****/
int c_break (void);
int ascending_sort (const void *p1, const void *p2);
int descending_sort (const void *p1, const void *p2);

/***********************************************************************/

void main ( )
{
    int i, j, k, ret;

    char buffer[81];
    LCB    *demo_LCB = NULL;
    char   *demo_LCB_buffer = NULL;

    char number_list[12][10] =  { "Zero",  "One",   "Two",
			          "Three", "Four",  "Five",
			          "Six",   "Seven", "Eight",
			          "Nine",  "Ten",   ""
			        };
    HARD_DISK_HEADROOM = 50000L;
    RAM_HEADROOM = 90000L;

    ctrlbrk (c_break);
    init_vmm ();

    for ( i=0; ret >= 0 && i<=10; i++ )
	ret = makeLIST ( demo_LCB, VM_DEFAULT,
			 strlen(number_list[i]), number_list[i] );

    if ( ret < 0 )     /* arggh! something failed in the makeLIST */
    {
	printf ( "\n\nmakeLIST failed with return code %d\n\n", ret );
        close_vmm();	/* release ems/xms if siezed by init_vmm() */
	exit ( -1 );
    }

    printf ( "Records are now in virtual memory:\n" );
    for ( i=ret=0; ret >=0 && i<=10; i++ )
	printf ( "\tRecord %d: %s\n", i, getLIST ( demo_LCB, i ) );

    if ( ret < 0 )
    {
	printf ( "\n\ngetLIST failed with return code %d\n\n", ret );
        close_vmm();	/* release ems/xms if siezed by init_vmm() */
	exit ( -1 );
    }
    printf ( "Strike any key to continue... " );
    getch();

    printf ( "\n\nOnce committed to the list, records can\n"
	     "be retrieved directly, in any order. For example:\n"
	     "    The firing order of the 1956 chevy 265 engine\n"
	     "    is 1-8-4-3-6-5-7-2:\n\n" );
    printf ( "Strike any key to continue... " );
    getch();

    printf ( "\n%s... ", getLIST ( demo_LCB, 1 ) );
    printf ( "%s... ", getLIST ( demo_LCB, 8 ) );
    printf ( "%s... ", getLIST ( demo_LCB, 4 ) );
    printf ( "%s... ", getLIST ( demo_LCB, 3 ) );
    printf ( "%s... ", getLIST ( demo_LCB, 6 ) );
    printf ( "%s... ", getLIST ( demo_LCB, 5 ) );
    printf ( "%s... ", getLIST ( demo_LCB, 7 ) );
    printf ( "%s...\n", getLIST ( demo_LCB, 2 ) );

    printf ( "\nStrike any key to sort the list alphabetically... " );
    getch();

    sortLIST(demo_LCB, NULL);

    printf ( "\n\nRecords are now sorted in ASCENDING (default) sequence.\n" );
    printf ( "Note that Zero sorts last because 'Z' collates\n" );
    printf ( "after all other letters in the alphabet:\n\n" );

    for ( i=0; i<=10; i++ )
	printf ( "\trecord %d: %s\n", i, getLIST ( demo_LCB, i ) );

    printf ( "\n\nStrike any key to use a binary search to\n" );
    printf ( "find the record 'three'... " );
    getch();

    ret = bsearchLIST ( demo_LCB, "three", 0,
		demo_LCB->next_index, 0, 5, ASCENDING_SEQ );

    if ( ret == -1 )
        printf ( "\n\nRecord not found!\n\n" );
    else
        printf ( "\n\nRecord %d is %s\n\n", ret,
        		getLIST ( demo_LCB, ret ));

    printf ( "\n\nStrike any key to sort the list in DESCENDING sequence... " );
    getch();

    sortLIST(demo_LCB, descending_sort);

    printf ( "\n\nList records are now sorted in DESCENDING sequence.\n" );
    printf ( "Note that Zero now sorts FIRST because of the results\n" );
    printf ( "of the comparison in 'descending_sort(...)':\n\n" );

    for ( i=0; i<=10; i++ )
	printf ( "\tRecord %d: %s\n", i, getLIST ( demo_LCB, i ) );

    printf ( "\n\nStrike any key to use a linear search to\n" );
    printf ( "find the record 'five'... " );
    getch();

    ret = lsearchLIST ( demo_LCB, "five", 0,
    		demo_LCB->next_index,0,5 );

    if ( ret == -1 )
        printf ( "\n\nRecord not found!\n\n" );
    else
        printf ( "\n\nRecord %d is %s\n\n", ret,
        		getLIST ( demo_LCB, ret ));

    printf ( "Strike any key to continue... " );
    getch();

    printf ( "\n\nThe list will now be 'torn down' and rebuilt.\n" );
    printf ( "We'll then look at how to modify list entries.\n\n" );
    printf ( "Strike any key to continue... " );
    getch();

    zapLIST (demo_LCB);

    for ( i=0; ret >= 0 && i<=10; i++ )
	ret = makeLIST ( demo_LCB, ON_DISK,
			strlen(number_list[i]), number_list[i] );

    if ( ret < 0 )
    {
	printf ( "\n\nmakeLIST failed with return code %d\n\n", ret );
        close_vmm();	/* release ems/xms if siezed by init_vmm() */
	exit ( -1 );
    }

    printf ( "\n\nThe list is rebuilt. We'll now translate\n" );
    printf ( "all the list elements to lower case letters.\n" );
    printf ( "\n\nRecords must be the same length (or shorter) to be\n" );
    printf ( "rewritten. Remember that YOU determine the length of\n" );
    printf ( "the record when its initially written. By anticipating\n" );
    printf ( "rewrites, you can ask that the record length be large\n" );
    printf ( "enough to rewrite when you need to...\n\n" );
    printf ( "Strike any key to continue... " );
    getch();

    for ( i=0; ret >= 0 && i<=10; i++ )
    {
	demo_LCB_buffer = getLIST ( demo_LCB, i );
	for ( j=0; *(demo_LCB_buffer+j); j++ )
	    *(demo_LCB_buffer+j) = tolower (*(demo_LCB_buffer+j));

	ret = updateLIST ( demo_LCB,	/* <- LCB		*/
			   i,			/* <- which record 	*/
			   demo_LCB_buffer,	/* <- where's the data? */
			   strlen(demo_LCB_buffer) ); /* how big?	*/
	if ( !ret )
	{
	    printf ( "\n\nupdateLIST failed" );
            close_vmm();
	    exit ( -1 );
	}
    }

    printf ( "\n\nRecords have been translated to lower case\n" );
    printf ( "and 're-written' by the list manager:\n\n" );

    for ( i=ret=0; ret >=0 && i<=10; i++ )
	printf ( "\tRecord %d: %s\n", i, getLIST ( demo_LCB, i ) );

    if ( ret < 0 )
    {
	printf ( "\n\ngetLIST failed with return code %d\n\n", ret );
        close_vmm();
	exit ( -1 );
    }

    sprintf ( buffer, "%s\\mylist.vmx", VMDISK );
    if ( !access ( buffer, 0 ) )    /* 0 = exists */
    {
        unlink  ( buffer );
        sprintf ( buffer, "%s\\mylist.vmd", VMDISK );
        unlink  ( buffer );
    }

    printf ( "\nStrike any key to save the list to disk... " );
    getch();

    saveLIST(demo_LCB, "mylist");
    
    printf ( "\n\n\The list has now been written out to disk.\n\n" );
    printf ( "Strike any key to read it back in from disk... " );
    getch();

    demo_LCB = restoreLIST ( "mylist", LIST_READ_ONLY );
    printf ( "\n\The list has now been brought in from disk.\n" );
    for ( i=0; i<=10; i++ )
	printf ( "\tRecord %d: %s\n", i, getLIST ( demo_LCB, i ) );

    
    printf ( "\nWell, those are the list manager functions. We hope\n" );
    printf ( "you find them valuable in managing large lists of records.\n\n" );
    printf ( "If you're interested in an unrestricted copy of this\n" );
    printf ( "library, please send in the order form in this directory.\n\n" );
    printf ( "Strike any key to exit... " );
    getch();

    printf ( "\n" );
    close_vmm();
    exit (0);
}

/**************** s o r t i n g   r e c o r d s ********************/

int ascending_sort (const void *p1, const void *p2)
{
    return(strcmp((char *)p1, (char *)p2));
}

/***********************************************************************/

int descending_sort (const void *p1, const void *p2)
{
    return (strcmp((char *)p2, (char *)p1));
}

/***********************************************************************/

int c_break(void)
{
   close_vmm();
   printf("Control-Break pressed.  Program aborting ...\n");
   return (ABORT);
}
