/* NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE */
/*									 */
/*     IF YOU RECOMPILE THIS PROGRAM USING THE DEMO XLLIST.LIB, IT WILL  */
/*     CEASE TO OPERATE PROPERLY BECAUSE THE DEMO LIBRARY LIMITS THE     */
/*     NUMBER OF LIST ELEMENTS YOU CAN BUILD TO 50... AND THIS DEMO IS   */
/*     BASED ON CREATING AND OPERATING ON 1000 RECORDS.                  */
/*									 */
/*     WE STRONGLY SUGGEST YOU RECOMPILE LISTDEMO.PRJ AND PLAY WITH IT!  */
/*     IT DEMONSTRATES THE SAME FEATURES WITH 10 ELEMENTS.  THIS PROGRAM */
/*     IS ESSENTIALLY A PROGRAM TO SHOW TIMING, AND THE SOURCE IS HERE   */
/*     SO YOU CAN GET AN IDEA OF WHAT IS GOING ON. 			 */
/*									 */
/* NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE */


#define MAIN

#include "xllist.h"
#include <time.h>

LCB *mylist;
char str[30];

#define ABORT 0

BITMAP create_bitmap(unsigned long num_elements)
{
    BITMAP b = NULL;

    /* if the requested number of elements is within range allocate the
       bitmap by dividing the number of elements by 8 and adding one for
       the times when num_elements is not exactly divisible by 8 */
    if (num_elements <= BITMAP_MAX && num_elements > 0)
	b = (BITMAP) calloc ((size_t )(num_elements >> 3) + 1, sizeof(BYTE));

    return (b);
}


int c_break(void)
{
   close_vmm();
   printf("Control-Break pressed.  Program aborting ...\n");
   return (ABORT);
}

void main()
{
    clock_t cstart, cend;
    BITMAP bitmap = NULL;
    int num, i, j;

    ctrlbrk(c_break);

    bitmap = create_bitmap(1000);	/* create a 1000-element bit map  */

    unlink  ( "mylist.vmd" );	/* make sure mylist.vmx & mylist.vmd */
    unlink  ( "mylist.vmx" );   /* don't exist...		     */

    cstart = clock();
    init_vmm();
    cend = clock();
    printf("Init time was: %f seconds\n", (cend - cstart) / CLK_TCK);

	/***************************************************************/
	/***************************************************************/
	/*****                                                     *****/
	/***** this is a crude way to generate 1000 pretty random  *****/
	/***** numbers so we can make a random list. We do it here *****/
	/***** to illustrate the sort which follows. Once we get   *****/
	/***** the numbers 0 - 1000 in random order, we'll sort    *****/
	/***** them into sequence using a "quicksort".		   *****/
	/*****                                                     *****/
	/***** On a 386-25mhz laptop, 1000 records sort in less    *****/
	/***** than 8 seconds...                                   *****/
	/*****                                                     *****/
	/***************************************************************/
	/***************************************************************/

    cstart = clock();
    for (i = 0; i < 2500; i++)
    {
	j = random(1000);		/* here we take 2500 stabs at     */
	if ( GET_BIT(bitmap, j) )	/* finding 1000 distinct, random  */
	    continue;			/* numbers. If we find a number   */
	    				/* has been used, try, try again. */

	SET_BIT (bitmap, j);		/* so we don't repeat this number */

	sprintf(str, "This is string %d", j);
	num = makeLIST(mylist, VM_DEFAULT, strlen(str), str);
	if ( num < 0 )
	{
	    printf ( "makeLIST failed w/return code %d\n", num );
	    printf ( "mylist->next_index == %d\n\n", mylist->next_index );
	    close_vmm ( );
	    exit ( num );
	}
    }
    
    for ( i=0; i<1000; i++ )
	if ( !GET_BIT(bitmap, i) )	/* fill up empty numbers so we  */
	{				/* have one each number between */
	    SET_BIT (bitmap, i);
	    sprintf(str, "This is string %d", i);
	    makeLIST(mylist, VM_DEFAULT, strlen(str), str);
	}
    
    cend = clock();
    printf( "Time to make 1000 RANDOM records in VIRTUAL MEMORY "
    	    "was: %f seconds\n", (cend - cstart) / CLK_TCK);

    printf( "Now, sort 1000 RANDOM records\r" );
    cstart = clock();
    sortLIST(mylist, NULL);
    cend = clock();
    printf("Sort 1000 RANDOM records: %f seconds\n",
    					(cend - cstart) / CLK_TCK);

	/***************************************************************/
	/***************************************************************/
	/*****                                                     *****/
	/***** Now we're going to use the binary search on the     *****/
	/***** sorted records. First we'll search for a single     *****/
	/***** record - usually that happens in +/- 1 clock tick,  *****/
	/***** so it isn't unusual to see that it took 0.0000 sec. *****/
	/***** that just means we found it in < 1 tick... and the  *****/
	/***** clock only 'ticks' 18.2 times / sec.		   *****/
	/*****                                                     *****/
	/***** Then we're going to do 100 binary searches. The	   *****/
	/***** average of the 100 is a better picture of how long  *****/
	/***** it takes to find a record using the binary search   *****/
	/***** algorythm.					   *****/
	/*****                                                     *****/
	/***************************************************************/
	/***************************************************************/

    sprintf(str, "This is string %d", 0);	
    cstart = clock();
    num = bsearchLIST(mylist, str, 0, 999, 0, strlen(str), ASCENDING_SEQ);
    cend = clock();
    printf("bsearchLIST time was: %f seconds\n", (cend - cstart) / CLK_TCK);

    cstart = clock();
    for (i = 0; i < 100; i++)
    {
	sprintf(str, "This is string %d", random(1000));	
	num = bsearchLIST(mylist, str, 0, 999, 0, strlen(str), ASCENDING_SEQ);
    }
    cend = clock();
    printf("100 Random Bsearch time in VIRTUAL MEMORY was: %f seconds\n",
    					(cend - cstart) / CLK_TCK);
    printf("Avg Bsearch time: %f seconds\n", ((cend - cstart) / CLK_TCK) / 100);


    cstart = clock();
    zapLIST(mylist);
    cend = clock();
    printf("Time to release resources was: %f seconds\n",
    						(cend - cstart) / CLK_TCK);

	/***************************************************************/
	/***************************************************************/
	/*****                                                     *****/
	/***** Using RAM is very fast... EMS is nearly as fast...  *****/
	/***** XMS is a little slower, and DISK is Sloooowwww...   *****/
	/*****   ... but not TOOOOO sloooow.                       *****/
	/*****                                                     *****/
	/***** The following example writes 1000 entries to a list *****/
	/***** on disk, searches for 100 records at random, saves  *****/
	/***** the list off for later retrieval, then picks up the *****/
	/***** list again...					   *****/
	/*****                                                     *****/
	/***************************************************************/
	/***************************************************************/

/* In order to save a list to disk, it must first be created ON_DISK... */

    cstart = clock();
    for (i = 0; i < 1000; i++)
    {
	sprintf(str, "This is string %d", i);
	num = makeLIST(mylist, ON_DISK, strlen(str), str);
	if ( num < 0 )
	{
	    printf ( "makeLIST failed w/return code %d\n", num );
	    printf ( "mylist->next_index == %d\n\n", mylist->next_index );
	    close_vmm ( );
	    exit ( num );
	}
    }
    cend = clock();
    printf("Time to make 1000 records ON DISK was: %f seconds\n",
    					(cend - cstart) / CLK_TCK);
    cstart = clock();
    for (i = 0; i < 100; i++)
    {
	sprintf(str, "This is string %d", random(1000));	
	num = bsearchLIST(mylist, str, 0, 999, 0, strlen(str), ASCENDING_SEQ);
    }
    cend = clock();
    printf("100 Random Bsearch time ON DISK was: %f seconds\n",
    					(cend - cstart) / CLK_TCK);
    printf("Avg Bsearch time: %f seconds\n", ((cend - cstart) / CLK_TCK) / 100);

    cstart = clock();
    saveLIST(mylist, "mylist");
    cend = clock();
    printf("Save time (to disk) was: %f seconds\n", (cend - cstart) / CLK_TCK);

    cstart = clock();
    mylist = restoreLIST("mylist", LIST_READ_ONLY);
    cend = clock();
    printf("Restore time (from disk) was: %f seconds\n", (cend - cstart) / CLK_TCK);

    cstart = clock();
    close_vmm();
    cend = clock();
    printf("Close time was: %f seconds\n", (cend - cstart) / CLK_TCK);
}
