/*
** Program:	NATLOG.C
** Purpose:	This program calculates e (the base of natural logarithms)
**		to 5000 decimal places. Due to it's compute intensive nature
**		and the size of it's computation workarea, it can be a good
**		benchmark for CPU speed.
** Based On:	NATLOG.MAC (1987 - Z80 CP/M assembly code)
**
** Author:	Lew Pitcher (1992)
**
**		This program released into the public domain.
*/

#include <stdio.h>

#define	I	1		/* number of bytes in integer portion	*/
#define	X	2096		/* number of bytes in fraction portion	*/
#define	N	1791		/* number of divide iterations		*/
#define	P	5000		/* number of multiply iterations	*/

main()
{
	char e[I+X];		/* holds fixed point binary value of e	*/
	unsigned n,		/* divisor for calculation loop		*/
		 p;		/* precision for print loop		*/

	printf("Computing \'e\' - Please stand bye.....\n\n");

	e[0] = 1;
	for (n = N; n > 0; --n)
	{
		divide(e,n,I+X);
		e[0] += 1;
	}

	fputs(" e == ",stdout); fputc(e[0]+'0',stdout); fputc('.',stdout);
	for (p = 0; p < P; ++p)
	{
		e[0] = 0;
		multiply(e,10,I+X);
		fputc(e[0]+'0',stdout);
		if (p%5 == 4)
			if (p%50 == 49)
				fputs("\n\t",stdout);
			else
				fputc(' ',stdout);
	}
	fputc('\n',stdout);
}

/*
** fixed point multi-precision division
*/
divide(dvd,dvr,prc)
char *dvd;
unsigned dvr,prc;
{
	char *Wk_q;
	unsigned bit, rmdr;

	rmdr = 0;
	for (Wk_q = dvd; Wk_q < dvd+prc; ++Wk_q)
	{
		for(bit = 0; bit < 8; ++bit)
		{
			rmdr *= 2;
			if (*Wk_q & 0200) rmdr += 1;
			*Wk_q <<= 1;
			if (rmdr >= dvr)
			{
				*Wk_q += 1;
				rmdr -= dvr;
			}
		}
	}
}

/*
** fixed point multi-precision multiplication
*/
multiply(mcd,mpy,prc)
char *mcd;
unsigned mpy, prc;
{
	char *Wk_r;
	unsigned bit, ovflo;

	ovflo = 0;
	for (Wk_r = mcd+prc-1; Wk_r >= mcd; --Wk_r)
	{
		for (bit = 8; bit > 0; --bit)
		{
			if (*Wk_r & 1) ovflo += mpy;
			*Wk_r >>= 1; *Wk_r &= 0177;
			if (ovflo & 1) *Wk_r = *Wk_r | 0200;
			ovflo /= 2;
		}
	}
}
