/*
  Routine to trap Ctrl-C and Ctrl-Break...eliminates the echo
  of ^C that occurs otherwise
*/

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <dos.h>
#include <litecomm.h>
#include "qbttl.h"

#ifdef M_I86
#pragma check_stack( off )
#pragma check_pointer( off )
#pragma intrinsic( _enable, _disable )
#endif

#define  BREAK_CHAR  0x0000   /* scan code and character for ctrl-break */
#define  CTRLC_CHAR  0x2e03   /* scan code and character for ctrl-c		*/

#ifdef __TURBOC__
static void interrupt (*old09)(void);	/* old hardware keyboard interrupt	*/
static void interrupt (*old1b)(void);	/* old BIOS break interrupt			*/

static void interrupt new09(void);		/* new hardware keyboard handler	*/
static void interrupt new1b(void);		/* new BIOS break handler			*/
#endif

#ifdef M_I86
static void (_interrupt _far *old09)(void);	/* old hardware keyboard interrupt	*/
static void (_interrupt _far *old1b)(void);	/* old BIOS break interrupt			*/

static void _interrupt _far new09(void);		/* new hardware keyboard handler	*/
static void _interrupt _far new1b(void);		/* new BIOS break handler			*/
#endif

char  far * bios_break_flag = (char far *) 0x471;

static int   kbd_buf_start;	/* offset address of start of BIOS keyboard buffer */
static int   kbd_buf_end;		/* offset address of end   of BIOS keyboard buffer */

/*
   New interrupt handler for the hardware keyboard interrupt.
   Calls the original handler and then inspects the keyboard
	  buffer to see if ^C or ctrl-break has been hit.  If so,
	  removes the offending character from the buffer.
*/
#ifdef __TURBOC__
void interrupt new09(void)
#endif
#ifdef M_I86
void _interrupt _far new09(void)
#endif
{
   int	 last_char;  /* last character entered into buffer	 */
   int	 tmp;		 /* temporary offset address into buffer */

   /*
	  Pointers to the low memory BIOS addresses that contain
		 the offset addresses of the current head and tail
		 positions in the BIOS buffer.
   */
   static int far * kbd_head = (int far *) 0x41a;
   static int far * kbd_tail = (int far *) 0x41c;

   (*old09)();	  /* call the old handler to do the really dirty work */

#ifdef __TURBOC__
	disable();
#endif
#ifdef M_I86
	_disable();
#endif

   if(*kbd_head != *kbd_tail)	 /* make sure something really there */
	  {
	  tmp = *kbd_tail - 2;		 /* point back to last character */

	  if(tmp < kbd_buf_start)	 /* see if backed up past buffer start */
		 tmp = kbd_buf_end - 2;  /* reset to last entry in buffer	   */

	  /*
		 Fetch character and scan code from buffer.
	  */
	  last_char = * (int far *) MK_FP(0x0040, tmp);

	/*
		 Remove a ^C or ctrl-break from buffer.
	*/
	  if(last_char == CTRLC_CHAR  ||  last_char == BREAK_CHAR)
		 {
		 *kbd_tail = tmp;		 /* set tail pointer to prev. calc. tmp. */
		 ctlc_hit = 1;		 /* set our private flag				 */
		 *bios_break_flag = 0;	 /* keep the BIOS flag clean			 */
		 }
	  }

#ifdef __TURBOC__
   enable();
#endif
#ifdef M_I86
	_enable();
#endif
}

/*
   Do nothing routine for BIOS break interrupt handler.
   Needed because DOS grabs this vector (typically) and
	  we call the old hardware keyboard handler before we
	  stomp breaks, and this interrupt is called from there.
*/
#ifdef __TURBOC__
void  interrupt   new1b(void)  { }
#endif
#ifdef M_I86
void  _interrupt _far new1b(void)	{ }
#endif

/*
   Initialize the break handling mechanism.
*/

void init_kbcap(void)
{
   /*
	  Pointers to BIOS locations wherein are stored the keyboard buffer
		 start and end (actually, 1 past) values.  NOT ALL BIOSs IMPLEMENT
		 THESE !!!	If if looks like there is nothing there, this routine
		 sets the standard default values.
   */
   int far * kbd_buf_start_ptr = (int far *) 0x480;
   int far * kbd_buf_end_ptr   = (int far *) 0x482;

#ifdef __TURBOC__
   old09 = getvect(0x09);  /* grab old hardware keyboard interrupt	 */
   old1b = getvect(0x1b);  /* grab old BIOS keyboard break interrupt */
#endif
#ifdef M_I86
   old09 = _dos_getvect(0x09);	/* grab old hardware keyboard interrupt	 */
   old1b = _dos_getvect(0x1b);	/* grab old BIOS keyboard break interrupt */
#endif

#ifdef __TURBOC__
	disable();
#endif
#ifdef M_I86
	_disable();
#endif

#ifdef __TURBOC__
   setvect(0x09, new09);   /* install our own hardware keyboard handler   */
   setvect(0x1b, new1b);   /* install our own BIOS keyboard break handler */
#endif
#ifdef M_I86
   _dos_setvect(0x09, new09);	/* install our own hardware keyboard handler   */
   _dos_setvect(0x1b, new1b);	/* install our own BIOS keyboard break handler */
#endif

#ifdef __TURBOC__
   enable();
#endif
#ifdef M_I86
	_enable();
#endif

   atexit(clear_kb_int);	   /* register a clean-up procedure */

   *bios_break_flag = 0;   /* start out with a clean flag	*/

   /*
	  Check start and end pointers for minimum validity
		 (both non-zero).  Set start and end offsets.
   */
   if( (*kbd_buf_start_ptr | *kbd_buf_end_ptr)	!=	0 )
	  {
	  kbd_buf_start = *kbd_buf_start_ptr;	 /* use what's there  */
	  kbd_buf_end	= *kbd_buf_end_ptr;
	  }
   else
	  {
	  kbd_buf_start = 0x1e;					 /* else use standard */
	  kbd_buf_end	= 0x3e;
	  }
}

/*
   Minimal clean-up.

   A critical error handler should be installed to make sure that
	  this clean-up procedure does not get bypassed.
*/
void  clear_kb_int(void)
{
#ifdef __TURBOC__
   disable();  /* so as not to break up a sequence */
#endif
#ifdef M_I86
   _disable();	/* so as not to break up a sequence */
#endif

#ifdef __TURBOC__
   setvect(0x09, old09);   /* restore hardware keyboard handler   */
   setvect(0x1b, old1b);   /* restore BIOS keyboard break handler */
#endif
#ifdef M_I86
   _dos_setvect(0x09, old09);	/* restore hardware keyboard handler   */
   _dos_setvect(0x1b, old1b);	/* restore BIOS keyboard break handler */
#endif

#ifdef __TURBOC__
   enable();   /* re-enable interrupts */
#endif
#ifdef M_I86
   _enable();	/* re-enable interrupts */
#endif

   *bios_break_flag = 0;   /* keep the flag clean */
}
