/*
**	QBTTL is a complete, if somewhat limited, terminal emulation
**	program  designed to demonstrate the use of the LiteComm(tm)
**	ToolBox.  The executable version is included so that you can
**	try it out while viewing the code.	To successfully create a
**	new version of QBTTL, you must have the protocol engines
**	which are provided as part of your registration package.
**
**	QBTTL functions as a limited ANSI terminal.
**	It will run on any communications port from
**	1 thru 4, defaulting to port 2 (COM2).	To execute QBTTL on other than
**	COM2, specify
**	  QBTTL n
**	where n is a number from 1 to 4.
**
**	Please note also that, to send a Ctrl-C, you must use the Alt-C keys
**
**	Information Technology, Ltd.
*/

#include "litecomm.h"
#include "lcproto.h"
#include "lcpxm.h"
#include "lczm.h"
#include "qbttl.h"

#include <stdlib.h>

#ifdef __TURBOC__
#include <conio.h>
#include <dir.h>
#include <dos.h>
#include <fcntl.h>
#include <sys\stat.h>
#include <io.h>
#include <mem.h>
#include <signal.h>
#include <string.h>
#endif

#ifdef M_I86
#include <signal.h>
#include <conio.h>
#include <ctype.h>
#include <dos.h>
#include <fcntl.h>
#include <graph.h>
#include <stdio.h>
#include <io.h>
#include <string.h>
#include <sys\types.h>
#include <sys\stat.h>

#endif

#ifdef __TURBOC__
unsigned _stklen = 8192;
#endif

#define CTRLX 0x18

#define ESC 0x1b
#define ENQ 0x05

void selproto(void);
void terminal(void);
void chgcomm(void);
void procesc(void);
void moveleft(void);
void moveright(void);
void moveup(void);
void movedn(void);
void show_modem(unsigned char);
void simtab();
void setcursorpos(void);
void send(void);
void recv(void);
void xupload(void);
void xdnload(void);
void wupload(void);
void wdnload(void);
void yzupload(void);
void yzdnload(void);
int kb_abort(void);
void yz_status(int, long, void *);
void xm_status(int, long, void *);
int strblank(char *);
char *strbtrim(char *);
void wperror(char *);
void wpinfo(char *);
void frame(char *);
void clreos(void);
void wputs(char *);
void wputc(int);

#ifdef M_I86
void clreol(void);
void gettextinfo(struct text_info *);
void textattr(int);
#endif

/*
** Buffer for string output - needed by MSC
*/
static char Buffer[81] = {0};

/*
**	Comm parameters - human-readable form
*/
long baud = 2400L;
char parity = 'N';
int data = 8;
int stop = 1;

/*
**	Comm parameters - Litecomm form
*/
long pbaud = 2400;
unsigned pparity = NPARITY;
unsigned pbits = BIT8;
unsigned pstop = STOP1;
unsigned protocol = XMODEM;
unsigned port = 1;

int xmode = 0;							/* xmodem type */
int yxmode = 0;							/* use small blocks */
int halfd = 0;							/* half-duplex */
int hostm = 0;							/* host mode */
int ctlc_hit = 0;

char MODEMSET0[] = "ATZ\r";
char MODEMSET1[] = "ATT E0\r";
char MODEMSET2[] = "ATC1 V0 X1 S0=1 M1\r";

/*
** protocol control block pointer - Rev 6 form
*/
PROTO *p;

int mbd;
int mbg;
int msay;
int mact;
int mnact;
int mtitle;
int sbd;
int sbg;
int ssay;
int sact;
int snact;
int stitle;
char	strbuf[80];					  /* for sprintf usage */

unsigned char *filelist[33];			/* file list - ZModem */

void main(argc, argv)
int argc;
char *argv[];
{
	int    opt;

	init_kbcap();						/* set up our KB handler */
/*
**	check for a port parameter
*/
	if (argc > 1)
	{
		port = atoi(argv[1]);
		if ((port < 1) || (port > 4))
		{
			puts("Invalid Port Specified\n");
			exit(4);
		}
	}

/*
** establish windowing environment
*/
#ifdef __TURBOC__
	_wscroll = 0;						/* disable scrolling */
#endif
#ifdef M_I86
	_wrapon(_GWRAPOFF);
#endif

	window(1, 1, 80, 25);
	clrscr();
	window(1, 1, 80, 24);

/*
** set-up the comm port
*/
	if (lc_open(port,2400,NPARITY,BIT8,STOP1,2048,8192,FALSE) == -1)
	{
		wperror("Can't open port");
		abort();
	}

/*
** allocate the protocol control block
*/
	if ((p = lcp_alloc(port)) == NULL)
	{
		wperror("Cant make proto control block");
		abort();
	}
	lcp_abort = kb_abort;

/*
** if this port is a 16550, activate the fifo
*/
	if (lc_get_type(port) == P16550A)
		lc_fifo_on(port);

	while (1)
	{
		clrscr();
		wputs("-- MAIN MENU --\r\n");
		wputs("T - enter Terminal mode\r\n");
		wputs("    Alt-X leaves terminal mode\r\n");
		sprintf(Buffer, "H - toggle Host mode (now %s)\r\n",hostm ? "ON":"OFF");
		wputs(Buffer);
		sprintf(Buffer, "G - toGgle half-duplex mode (now %s)\r\n",halfd ? "ON":"OFF");
		wputs(Buffer);
		sprintf(Buffer, "C - change Comm parameters (now %ld,%c,%d,%d)\r\n",
				baud,parity,data,stop);
		wputs(Buffer);
		wputs("P - Select protocol, currently ");
		switch(protocol)
		{
			case XMODEM:
				wputs("XModem\r\n");
				break;
			case XMODEM1K:
				wputs("1K XModem\r\n");
				break;
			case WXMODEM:
				wputs("Windowed XModem\r\n");
				break;
			case YMODEM:
				wputs("YModem\r\n");
				break;
			case ZMODEM:
				wputs("ZModem\r\n");
				break;
		}
		wputs("S - Send a file\r\n");
		wputs("R - Receive a file\r\n");
		wputs("Q - Quit\r\n\r\n");

		while (1)
			if ((opt = getch()) == 0x00)			/* extended char */
				getch();							/* ignore it */
			else
				break;
		opt = toupper(opt);

		switch (opt)
		{
			case 'T':	terminal();
						break;
			case 'H':	if (hostm)
							hostm = 0;
						else
						{
							hostm = 1;
							halfd = 0;
						}
						break;
			case 'G':	if (halfd)
							halfd = 0;
						else
						{
							halfd = 1;
							hostm = 0;
						}
						break;
			case 'P':	selproto();
						break;
			case 'S':	send();
						break;
			case 'R':	recv();
						break;
			case 'C':	chgcomm();
						break;
		}
		if (opt == 'Q')
			break;						/* shut down time */
	}									/* while (1) */

	if (lc_get_type(port) == P16550A)
		lc_fifo_off(port);

	lc_close(port,FALSE);
	lcp_free(p);						/* release proto control */
	window(1, 1, 80, 25);
	clrscr();
	exit(0);
}										/* main */

void terminal()
{
	int ch;
	int row, col;

	clrscr();

/*
** Force initial modem status display
*/
	show_modem (lc_mstat(port) | 0x0f);

	while (1)
	{
		ctlc_hit = FALSE;
		if ((ch = lc_get(port)) != -1)
			switch (ch & 0x7f)
			{
				case ESC:
					procesc();
					break;
				default:
					if (pparity != NPARITY)
						ch &= 0x7f;				/* strip parity */
					wputc(ch);
					if (hostm)
					{
						lc_put(port, ch);			/* echo back */
						if (ch == '\n')				/* was it return */
						{
							lc_put(port, '\n');		/* echo lf as well */
							wputc('\r');
							wputc('\n');
						}
					}
			}
		show_modem (lc_mstat(port));

		if (kbhit() || ctlc_hit)					   /* anything typed ? */
		{
			if (ctlc_hit)
				ch = 0x03;
			else
				if ((ch = getch()) == 0x00)		/* extended character ? */
				{
					if ((ch = getch()) == 0x2d) /* ALT-X ? */
						return;
					else
						continue;				/* throw away other extended */
				}
			lc_put(port, ch & 0x007f);		/* xmit the char */
			if (hostm || halfd)				/* local echo needed ? */
			{
				wputc(ch);
				if (ch == '\n')				/* was it CR */
				{
					wputc('\r');
					wputc('\n');	 /* add LF */
					if (hostm)
						lc_put(port, '\n'); /* and send LF in host mode */
				}
			}
		}
	}										/* while */
}

void procesc()
{
	int ch;


	ch = lc_getw(port);					/* wait for actual command */

	switch (ch & 0x7f)
	{
		case 'A':
			moveup();
			break;
		case 'B':
			movedn();
			break;
		case 'C':
			moveright();
			break;
		case 'D':
			moveleft();
			break;
		case 'H':
			gotoxy(1,1);
			break;
		case 'J':
			clreos();
			break;
		case 'K':
			clreol();
			break;
		case 'j':
			clrscr();
			break;
		case 'Y':
			setcursorpos();
			break;
	}
}										/* procesc */

void selproto()
{
	int opt;

	clrscr();
	wputs("-- SELECT PROTOCOL --\r\n");
	wputs("1 - XMODEM\r\n");
	wputs("2 - 1K XMODEM\r\n");
	wputs("3 - Windowed XMODEM\r\n");
	wputs("4 - YMODEM\r\n");
	wputs("5 - ZMODEM\r\n");

	while (1)
		if ((opt = getch()) == 0x00)			/* extended char */
			getch();							/* ignore it */
		else
			break;
	switch (opt)
	{
		case '1':
			protocol = XMODEM;
			break;
		case '2':
			protocol = XMODEM1K;
			break;
		case '3':
			protocol = WXMODEM;
			break;
		case '4':
			protocol = YMODEM;
			break;
		case '5':
			protocol = ZMODEM;
			break;
	}
}

/*
** The show_modem function displays the status of all the modem
** control signals that are normally asserted by the modem (DCE)
** It is not unusual for one or more of the signals to be absent,
** and is highly dependent on the type of connection, and the type
** of modem in use
*/
void show_modem(unsigned char mdmstat)
{
	struct text_info ti;					/* to store previous window */

	if ((mdmstat & 0x0f))			/* any status change ? */
	{
		gettextinfo(&ti);						/* get prev window data */
		window(1, 25, 80, 25);					/* set new window extents */
		textcolor(BLACK);						/* inverse the status line */
		textbackground(LIGHTGRAY);

		clrscr();							/* erase existing values */
		if (mdmstat & CTS)					/* Clear To Send */
		{
			gotoxy(3, 1);
			wputs("CTS");
		}
		if (mdmstat & DSR)					/* Data Set Ready */
		{
			gotoxy(9, 1);
			wputs("DSR");
		}
		if (mdmstat & DCD)					/* Carrier Detect */
		{
			gotoxy(15, 1);
			wputs("DCD");
		}
		if (mdmstat & RI)					/* Ring Indicate */
		{
			gotoxy(21, 1);
			wputs("RI");
		}
/*
** Put the previous window back the way it	was
*/
		window(ti.winleft, ti.wintop, ti.winright, ti.winbottom);
		textattr(ti.attribute);
		gotoxy(ti.curx, ti.cury);
	}
}										/* show_modem */

void setcursorpos()
{
	int ch;
	int x;
	int y;

	while((ch = lc_get(port)) == -1)
	;
	y = (ch & 0x7f) - 32;

	y++;									/* Turbo-C is rel to 1 */

	while((ch = lc_get(port)) == -1)
	;
	x = (ch & 0x7f) - 32;

	x++;									/* Turbo-C is rel to 1 */

	gotoxy(x, y);
}

void moveleft()
{
	int scurx,
		scury;
	struct text_info r;

	gettextinfo(&r);
	scurx = r.curx;
	scury = r.cury;

	scurx--;							 /* decrement x position */
	if (scurx < r.winleft)
	{
		scury--;							/* wrap to end of prev line */
		scurx = r.winright;
		if (scury < r.wintop);				/* wrap if at the top */
			scury = r.winbottom;
	}
	gotoxy(scurx, scury);
}										 /* moveleft */

void moveright()
{
	int scurx,
		scury;
	struct text_info r;

	gettextinfo(&r);
	scurx = r.curx;
	scury = r.cury;

	scurx++;							 /* decrement x position */
	if (scurx > r.winright)
	{
		scury++;						/* wrap to start of next line */
		scurx = r.winleft;
		if (scury > r.winbottom);		/* wrap to top if at bottom */
			scury = r.wintop;
	}
	gotoxy(scurx, scury);
}										 /* moveright */

void moveup()
{
	int scury;

	struct text_info r;

	gettextinfo(&r);
	scury = r.cury;

	scury = r.cury - 1;
	if (scury < r.wintop)
		scury = r.winbottom;				/* wrap to bottom */
	gotoxy(r.curx, scury);
}										/* moveup */

void movedn()
{
	int scury;

	struct text_info r;

	gettextinfo(&r);
	scury = r.cury;

	scury = r.cury + 1;
	if (scury > r.winbottom)
		scury = r.wintop;					/* wrap to top */
	gotoxy(r.curx, scury);
}

void simtab()
{
	int scurx,
		scury;
	struct text_info r;

	gettextinfo(&r);
	scurx = r.curx;
	scury = r.cury;

	do
		scurx++;						 /* bump x position */
	while (scurx % 9);

	if (scurx > r.winright)
	{
		scury++;
		scurx = r.winleft;
		if (scury > r.winbottom);
		{
			gotoxy(r.winright, r.winbottom);	/* force a scroll */
			wputc('\r');
			wputc('\n');
			return;
		}
	}
	gotoxy(scurx, scury);
}										 /* simtab */


void chgcomm()
{
	int opt;
	long sbaud;
	unsigned sparity;
	unsigned sbits;
	unsigned sstop;
	long hbaud;
	char hparity;
	int hdata;
	int hstop;

/*
** get current default settings
*/
	sbaud = pbaud;
	sparity = pparity;
	sbits = pbits;
	sstop = pstop;
	hbaud = baud;
	hparity = parity;
	hdata = data;
	hstop = stop;


	while (1)
	{
		clrscr();
		wputs("-- COMM PARAMETERS --\r\n");
		sprintf(Buffer,"  (Presently %ld,%c,%d,%d)\r\n",
				hbaud,hparity,hdata,hstop);
		wputs(Buffer);
		wputs("B - change Baud Rate\r\n");
		wputs("P - change Parity\r\n");
		wputs("D - change Data bits\r\n");
		wputs("S - change Stop bits\r\n\r\n");
		wputs("A - Abandon Changes\r\n");
		wputs("Q - Quit to Main Menu\r\n\r\n");

		while (1)
			if ((opt = getch()) == 0x00)			/* extended char */
				getch();							/* ignore it */
			else
				break;
		opt = toupper(opt);

		switch (opt)
		{
			case 'A':
				return;
			case 'Q':
				pbaud = sbaud;			/* reset the globals */
				pparity = sparity;
				pbits = sbits;
				pstop = sstop;
				baud = hbaud;
				parity = hparity;
				data = hdata;
				stop = hstop;
				comm_setup(port,pbaud,pparity,pbits,pstop);
				return;
			case 'B':
				wputs("1 - 600, 2 - 1200, 3 - 2400\r\n");
				wputs("4 - 4800, 5 - 9600, 6 - 19200\r\n");
				wputs("7 - 38400, 8 - 57600, 9 - 115200\r\n");

				while (1)
					if ((opt = getch()) == 0x00)			/* extended char */
						getch();							/* ignore it */
					else
						break;

				switch (opt)
				{
					case '1': hbaud = 600L;
							  sbaud = 600L;
							  break;
					case '2': hbaud = 1200L;
							  sbaud = 1200L;
							  break;
					case '3': hbaud = 2400L;
							  sbaud = 2400L;
							  break;
					case '4': hbaud = 4800L;
							  sbaud = 4800L;
							  break;
					case '5': hbaud = 9600L;
							  sbaud = 9600L;
							  break;
					case '6': hbaud = 19200L;
							  sbaud = 19200L;
							  break;
					case '7': hbaud = 38400L;
							  sbaud = 38400L;
							  break;
					case '8': hbaud = 57600L;
							  sbaud = 57600L;
							  break;
					case '9': hbaud = 115200L;
							  sbaud = 115200L;
							  break;
				}
				break;
			case 'P':
				wputs("1 - NONE, 2 - EVEN, 3 - ODD, 4 - MARK, 5 - SPACE\r\n");
				while (1)
					if ((opt = getch()) == 0x00)			/* extended char */
						getch();							/* ignore it */
					else
						break;
				switch (opt)
				{
					case '1': hparity = 'N';
							  sparity = NPARITY;
							  break;
					case '2': hparity = 'E';
							  sparity = EPARITY;
							  break;
					case '3': hparity = 'O';
							  sparity = OPARITY;
							  break;
					case '4': hparity = 'M';
							  sparity = MPARITY;
							  break;
					case '5': hparity = 'S';
							  sparity = SPARITY;
							  break;
				}
				break;
			case 'D':
				wputs("Number of Data Bits (5, 6, 7, 8)\r\n");
				while (1)
					if ((opt = getch()) == 0x00)			/* extended char */
						getch();							/* ignore it */
					else
						break;
				switch (opt)
				{
					case '5': hdata = 5;
							  sbits = BIT5;
							  break;
					case '6': hdata = 6;
							  sbits = BIT6;
							  break;
					case '7': hdata = 7;
							  sbits = BIT7;
							  break;
					case '8': hdata = 8;
							  sbits = BIT8;
							  break;
				}
				break;
			case 'S':
				wputs("Number of Stop Bits (1, 2)\r\n");
				while (1)
					if ((opt = getch()) == 0x00)			/* extended char */
						getch();							/* ignore it */
					else
						break;
				switch (opt)
				{
					case '1': hstop = 1;
							  sstop = STOP1;
							  break;
					case '2': hstop = 2;
							  sstop = STOP2;
							  break;
				}
				break;
		}
	}
}


void send()
{
	struct text_info ti;

	gettextinfo(&ti);						/* get prev window params */

	window(16, 7, 62, 12);					/* set the new window */
	textcolor(WHITE);
	textbackground(BLACK);					/* set the colors */
	frame("SEND A FILE");					/* add in a frame and title */
	window(17, 8, 60, 10);					/* work inside the frame */

	lcp_setproto(p, protocol, TRUE);
	switch (protocol)
	{
		case XMODEM:
		case XMODEM1K:
			lcp_status = xm_status;
			xupload();
			break;
		case WXMODEM:
			lcp_status = xm_status;
			wupload();
			break;
		case YMODEM:
			lcp_status = xm_status;
			yzupload();
			break;
		case ZMODEM:
			lcp_status = yz_status;
			yzupload();
			break;
	}

/*
** Put the previous window back the way it	was
*/
	window(ti.winleft, ti.wintop, ti.winright, ti.winbottom);
	textattr(ti.attribute);
	gotoxy(ti.curx, ti.cury);
}

void recv()
{
	struct text_info ti;

	gettextinfo(&ti);						/* get prev window params */

	window(16, 7, 62, 12);					/* set the new window */
	textcolor(WHITE);
	textbackground(BLACK);					/* set the colors */
	frame("RECEIVE A FILE");				/* add in a frame and title */
	window(17, 8, 60, 10);					/* work inside the frame */

	lcp_setproto(p, protocol, TRUE);
	switch (protocol)
	{
		case XMODEM:
		case XMODEM1K:
			lcp_status = xm_status;
			xdnload();
			break;
		case WXMODEM:
			lcp_status = xm_status;
			wdnload();
			break;
		case YMODEM:
			lcp_status = xm_status;
			yzdnload();
		case ZMODEM:
			lcp_status = yz_status;
			yzdnload();
			break;
	}

/*
** Put the previous window back the way it	was
*/
	window(ti.winleft, ti.wintop, ti.winright, ti.winbottom);
	textattr(ti.attribute);
	gotoxy(ti.curx, ti.cury);
}

void getfname(char *fname)
{
	char tmp[14];

	clrscr();
	wputs("File Name: ");
	tmp[0] = 12;							/* max of 12 char */
	cgets(tmp);
	wputs("\r\n");
	strcpy(fname, tmp+2);					/* copy into final name */
}

void getpname(char *fname)
{
	char tmp[82];

	clrscr();
	wputs("Storage Path: ");
	tmp[0] = 80;							/* max of 80 char */
	cgets(tmp);
	wputs("\r\n");
	strcpy(fname, tmp+2);					/* copy into final name */
}

void xupload(void)
{
	char fname[81];

	getfname(fname);

	if (strblank(fname))
		return;

	strbtrim(fname);

/*
** now switch the comport mode to 8 bits for xmodem protocol
*/
	if (comm_setup(port,pbaud,NPARITY,BIT8,pstop) == ERR)
	{
		wperror("Unable to switch line mode");
		return;
	}

	lcxm_send(p, fname);

	comm_setup(port,pbaud,pparity,pbits,pstop);
}

void xdnload(void)
{
	char fname[81];

	getfname(fname);
	if (strblank(fname))
		return;
	strbtrim(fname);

/*
** now switch the comport mode to 8 bits for xmodem protocol
*/
	if (comm_setup(port,pbaud,NPARITY,BIT8,pstop) == ERR)
	{
		wperror("Unable to switch line mode");
		return;
	}

	lcxm_recv(p, fname);

	comm_setup(port,pbaud,pparity,pbits,pstop);
}

void wupload(void)
{
	char fname[81];
	int  fd;
	int hmode;
	int result;
	int nrec;							/* returned record */

	ctlc_hit = FALSE;
	getfname(fname);
	if (ctlc_hit == TRUE)
	{
		ctlc_hit = FALSE;
		return;
	}
	if (strblank(fname))
		return;
	strbtrim(fname);

	if ((fd = open(fname,(O_RDONLY|O_BINARY))) == -1)
	{
		wperror("Unable to open the file");
		return;
	}
/*
** the file has been opened successfully...now switch the
** comport mode to 8 bits for xmodem protocol
*/
	if (comm_setup(port,pbaud,NPARITY,BIT8,pstop) == ERR)
	{
		wperror("Unable to switch line mode");
		return;
	}
	lc_xoff(port,TRUE);					   /* turn on auto xon-xoff */

/*
** begin file transmission to receiver -
** in receiver's specified mode
*/
	while (TRUE)
	{
		memset(p->buff, 0x1a, 128);		/* clear buff, short rec */
		if (read(fd, p->buff, 128) < 1)  /* EOF or Error */
			nrec = -1;				/* defines End of File */
		else
			nrec = 0;

		result = lcpwxsnd(p, &nrec);

		switch(result)			 /* what action to take ? */
		{
			case SUCCESS:
				if (nrec == -1)  /* EOF confirmed */
					wpinfo("End of Transmission");
				else
				{
					gotoxy(2,2);
					clreol();
					sprintf(Buffer,"Sent record: %d",p->rec);
					wputs(Buffer);
				}
				break;
			case CAN:
				wpinfo("Cancel Received");
				break;
			case RETRIES:
				sprintf(strbuf, "Too Many tries...Rec %d",p->rec);
				wperror(strbuf);
				break;
			case RESEND:				/* must reset file position */
				lseek(fd, (long)(nrec * 128), SEEK_SET);
				break;
			default:
				wperror("Fatal transmission error");
				break;
		}
		if (result == RESEND)
			continue;
		if (result == SUCCESS)
			if (nrec == -1)				/* was EOF signalled */
				break;
			else
				continue;
		break;							/* some other error */
	}										   /* while TRUE */
	lc_xoff(port, FALSE);						  /* turn off auto xon/xoff */
	comm_setup(port,pbaud,pparity,pbits,pstop);
	close(fd);								   /* close down the file */
}

void wdnload(void)
{
	char fname[81];
	int  fd;
	int result;

	ctlc_hit = FALSE;

	getfname(fname);
	if (ctlc_hit == TRUE)
	{
		ctlc_hit = FALSE;
		return;
	}
	if (strblank(fname))
		return;
	strbtrim(fname);

	if ((fd = open(fname,
		 (O_WRONLY|O_CREAT|O_TRUNC|O_BINARY),(S_IREAD|S_IWRITE))) == -1)
	{
		wperror("Unable to open the file");
		return;
	}
/*
** the file has been opened successfully...now switch the
** comport mode to 8 bits for xmodem protocol
*/
	if (comm_setup(port,pbaud,NPARITY,BIT8,pstop) == ERR)
	{
		wperror("Unable to switch line mode");
		return;
	}
/*
** setup conditions for WXMODEM receive
*/
	while (TRUE)
		{
		while ((result = lcpwxrec(p)) == SUCCESS)
		{
			write(fd, p->buff, 128);  /* dump the block */
			gotoxy(2,2);
			clreol();
			sprintf(Buffer,"Received record: %d", p->rec-1);
			wputs(Buffer);
		}
		switch(result)			 /* unusual conditions */
		{
			case DUPSEQ:
				gotoxy(2,2);
				clreol();
				wputs("Duplicate record seq");
				break;
			case CAN:
				wpinfo("Cancelled by host");
				break;
			case EOT:
				wpinfo("Normal Termination");
				break;
			case TOUT:
				wperror("SOH timeout");
				break;
			case RETRIES:
				sprintf(strbuf, "Too Many tries...Rec %d",(p->rec-1));
				wperror(strbuf);
				break;
			default:
				wperror("Fatal transmission error");
				break;
		}
		if ((result != SUCCESS) && (result != DUPSEQ))
			break;
	}										   /* while TRUE */

	comm_setup(port,pbaud,pparity,pbits,pstop);
	close(fd);								   /* close down the file */
}

void yzupload(void)
{
	char fname[81];
	int result;
	int i;
	int j;
	int nrec;							/* returned record */
#ifdef __TURBOC__
	struct ffblk fi;
#else
	struct find_t fi;
#endif

	ctlc_hit = FALSE;
	getfname(fname);
	if (ctlc_hit == TRUE)
	{
		ctlc_hit = FALSE;
		return;
	}
	if (strblank(fname))
		return;
	strbtrim(fname);

	i = 0;
#ifdef __TURBOC__
	result = findfirst(fname, &fi,0);
	while ((result != -1) && (i < 32))
#else
	result = _dos_findfirst(fname, 0, &fi);
	while ((result == 0) && (i < 32))
#endif
	{
#ifdef __TURBOC__
		filelist[i] = strdup(fi.ff_name);
#else
		filelist[i] = strdup(fi.name);
#endif
		if (filelist[i] == NULL)
		{
			for (j = 0; j < i; j++)
				free(filelist[j]);
			return;
		}
		i++;
#ifdef __TURBOC__
		result = findnext(&fi);
#else
		result = _dos_findnext(&fi);
#endif
	}
	filelist[i] = strdup("");					/* null name terminates list */

/* now switch the
** comport mode to 8 bits for xmodem protocol
*/
	if (comm_setup(port,pbaud,NPARITY,BIT8,pstop) == ERR)
	{
		wperror("Unable to switch line mode");
		for (j = 0; j < i; j++)
			free(filelist[j]);
		return;
	}

/*
** begin file transmission to receiver -
** in receiver's specified mode
*/
	if (protocol == YMODEM)
		lcym_send(p, (unsigned char**) filelist);				/* go do it */
	else
		lczm_send(p, (long) pbaud, (unsigned char **) filelist);

	comm_setup(port,pbaud,pparity,pbits,pstop); /* reset params */
	for (j = 0; j < i; j++)
		free(filelist[j]);
}

void yzdnload(void)
{
	char fname[81];
	int result;

	ctlc_hit = FALSE;

	getpname(fname);
	if (ctlc_hit == TRUE)
	{
		ctlc_hit = FALSE;
		return;
	}
	if (strblank(fname))
		return;
	strbtrim(fname);

/*
** the file has been opened successfully...now switch the
** comport mode to 8 bits for xmodem protocol
*/
	if (comm_setup(port,pbaud,NPARITY,BIT8,pstop) == ERR)
	{
		wperror("Unable to switch line mode");
		return;
	}
/*
** process the download
*/
	if (protocol == YMODEM)
		lcym_recv(p, fname);
	else
		lczm_recv(p, pbaud, fname);

	comm_setup(port,pbaud,pparity,pbits,pstop);
}

int kb_abort(void)
{
	char ch;

	if (kbhit())
	{
		while (1)
			if ((ch = getch()) == 0x00)			/* extended char */
				getch();							/* ignore it */
			else
				break;
		if (ch == 0x1b)					/* escape ? */
			return(TRUE);
	}
	return (FALSE);
}

void yz_status(int mcode, long intval, void *ptr)
{
	if ((mcode == ZSNDBLK) || (mcode == ZRECBLK))
		gotoxy(2,2);
	else
		gotoxy(2,3);
	clreol();
	switch (mcode)
	{
		case ZBADFILE:
			sprintf(Buffer,"Unable to open file %ld", intval);
			wputs(Buffer);
			break;
		case ZBADHDR:
			wputs("Bad header received");
			break;
		case ZBADPOS:
			wputs("Positioning Error");
			break;
		case ZDISKERR:
			sprintf(Buffer,"Disk error occurred at %ld", intval);
			wputs(Buffer);
			break;
		case ZFCREAT:
			wputs("Error trying to create file");
			break;
		case ZFOPEN:
			wputs("Error trying to open file");
			break;
		case ZINITTO:
			wputs("Initialization Timeout");
			break;
		case ZGOTFILE:
			sprintf(Buffer,"Receiving file %16.16s",((TELINK *)ptr)->filename);
			wputs(Buffer);
			break;
		case ZNOCAR:
			wputs("Loss of Carrier Detected");
			break;
		case ZPKTLEN:
			sprintf(Buffer,"Packet of length %ld received", intval);
			wputs(Buffer);
			break;
		case ZRECOVER:
			wputs("Attempting to restart transfer");
			break;
		case ZSEEKERR:
			wputs("Seek Error");
			break;
		case ZTIMER:
			wputs("TIMEOUT");
			break;
		case ZXFRCAN:
			wputs("Transfer Cancelled");
			break;
		case ZRECBLK:
			sprintf(Buffer,"Received %ld", intval);
			wputs(Buffer);
			break;
		case ZSNDFILE:
			sprintf(Buffer,"Sending file %s", filelist[(int)intval]);
			wputs(Buffer);
			break;
		case ZSNDBLK:
			sprintf(Buffer,"Sent %ld", intval);
			wputs(Buffer);
			break;
		default:
			sprintf(Buffer,"Unknown Message Code: %d", mcode);
			wputs(Buffer);
			break;
	}
}

void xm_status(int mcode, long intval, void *ptr)
{
	if (mcode == SUCCESS)
		gotoxy(2,2);
	else
		gotoxy(2,3);

	clreol();

	switch (mcode)
	{
		case XBADFILE:
			sprintf(Buffer,"Unable to open file %ld\r\n", intval);
			wputs(Buffer);
			break;
		case WAITACK:
			wputs("Waiting for ACK");
			break;
		case WAITBLK:
			wputs("Waiting for BLOCK");
			break;
		case WAITFNB:
			wputs("Waiting for FILENAME");
			break;
		case WAITSOH:
			wputs("Waiting for Start of Header");
			break;
		case XDISKERR:
			sprintf(Buffer,"Disk error occurred at %ld", intval);
			wputs(Buffer);
			break;
		case XGOTFILE:
			sprintf(Buffer,"Receiving file %16.16s\r\n",((TELINK *)ptr)->filename);
			wputs(Buffer);
			break;
		case TIMEOUT:
			wputs("TIMEOUT");
			break;
		case CAN:
			wputs("Transfer Cancelled");
			break;
		case DUPSEQ:
			sprintf(Buffer,"Duplicate Block %ld", intval);
			wputs(Buffer);
			break;
//		case EOT:
//			wputs("End of File");
//			break;
		case FATAL:
			sprintf(Buffer,"Fatal Error block %ld", intval);
			wputs(Buffer);
			break;
		case RETRIES:
			sprintf(Buffer,"Too Many Retries block %ld", intval);
			wputs(Buffer);
			break;
		case SUCCESS:
			sprintf(Buffer,"%ld Bytes Transferred", intval);
			wputs(Buffer);
			break;
		default:
			sprintf(Buffer,"Unknown Message Code: %d", mcode);
			wputs(Buffer);
			break;
	}
}

int strblank(char *str)
{
	char *tmp;

	for (tmp = str; ((*tmp) && (*tmp == ' ')); tmp++)
		;
	if (*tmp)
		return FALSE;
	return TRUE;
}

char *strbtrim(char *str)
{
	int len;

	while ((len = strlen(str)) && (*str == ' '))   /* trim leading spaces */
		memmove(str, str+1, len);

	while((len = strlen(str)) && ( *(str+len-1) == ' '))  /* trim trailing spaces */
		*(str+len-1) = 0x00;
	return str;
}

void wperror(char *str)
{
	struct text_info ti;
	char *swin;								/* pointer to save buffer */

#ifdef __TURBOC__
	if ((swin = (char *)malloc(500)) != NULL)
		gettext(16, 7, 61, 11, (char *)swin);	/* save underlying screen */
#endif

	gettextinfo(&ti);						/* get prev window params */

	window(16, 7, 62, 12);					/* set the new window */
	textcolor(WHITE);
	textbackground(RED);					/* set the colors */
	frame("ERROR");					/* add in a frame and title */
	window(17, 8, 60, 10);					/* work inside the frame */

	gotoxy(2, 3);
	wputs(str);
	while (1)
		if (getch() == 0x00)			/* extended char */
		{
			getch();							/* ignore it */
			break;
		}
		else
			break;
/*
** Put the previous window back the way it	was
*/
	window(ti.winleft, ti.wintop, ti.winright, ti.winbottom);
	textattr(ti.attribute);

#ifdef __TURBOC__
	if (swin != NULL)
	{
		puttext(16, 7, 61, 11, (char *)swin);	/* restore underlying screen */
		free(swin);
	}
#endif

	gotoxy(ti.curx, ti.cury);
}

void wpinfo(char *str)
{
	struct text_info ti;
	char *swin;								/* pointer to save buffer */

#ifdef __TURBOC__
	if ((swin = (char *)malloc(500)) != NULL)
		gettext(16, 7, 61, 11, (char *)swin);	/* save underlying screen */
#endif

	gettextinfo(&ti);						/* get prev window params */

	window(16, 7, 62, 12);					/* set the new window */
	textcolor(BLACK);
	textbackground(CYAN);					/* set the colors */
	frame("INFORMATION");					/* add in a frame and title */
	window(17, 8, 60, 10);					/* work inside the frame */

	gotoxy(2, 3);
	wputs(str);
	while (1)
		if (getch() == 0x00)			/* extended char */
		{
			getch();							/* ignore it */
			break;
		}
		else
			break;
/*
** Put the previous window back the way it	was
*/
	window(ti.winleft, ti.wintop, ti.winright, ti.winbottom);
	textattr(ti.attribute);

#ifdef __TURBOC__
	if (swin != NULL)
	{
		puttext(16, 7, 61, 11, (char *)swin);	/* restore underlying screen */
		free(swin);
	}
#endif

	gotoxy(ti.curx, ti.cury);
}

void frame(char *title)
{
	int i;
	int width, hite;
	int tlen;
	struct text_info ti;

	clrscr();								/* clear the window */

	gettextinfo(&ti);							/* see what the window looks like */
	width = ti.winright - ti.winleft;
	hite = ti.winbottom - ti.wintop;

	gotoxy(1, 1);
	wputc('');
	gotoxy(width, 1);
	wputc('');
	gotoxy(1, hite);
	wputc('');
	gotoxy(width, hite);
	wputc('');									/* causes scroll in last pos */
	for (i = 2; i < width; i++)
	{
		gotoxy(i, 1);
		wputc('');
		gotoxy(i, hite);
		wputc('');
	}
	for (i = 2; i < hite; i++)
	{
		gotoxy(1, i);
		wputc('');
		gotoxy(width, i);
		wputc('');
	}
#ifdef __TURBOC__
/*
** This code only works correctly under Turbo/Borland C -
** There is a bug in the MSC Graphics/Text routine
*/
	tlen = strlen(title);
	if (! tlen)
		return;
	gotoxy(3,1);
	wputs(title);
#endif
}

void clreos()
{
	struct text_info ti;
	int i;

	gettextinfo(&ti);
	clreol();
	for (i = ti.cury; i <= ti.winbottom; i++)
	{
		gotoxy(1, i);
		clreol();
	}
	gotoxy(ti.curx, ti.cury);
}

void wputs(char *str)
{
#ifdef __TURBOC__
	cputs(str);
#endif

#ifdef M_I86
	_outtext(str);
#endif
}

void wputc(int c)
{
#ifdef M_I86
	char s[2];
#endif
#ifdef __TURBOC__
	putch(c);
#endif

#ifdef M_I86
	s[0] = c;
	s[1] = 0;
	_outtext(s);
#endif
}

#ifdef M_I86
void gettextinfo(struct text_info *t)
{
	struct rccoord rc;
	short left, top, right, bottom;

	/* Save original foreground, background, and text position. */
	t->attribute = _gettextcolor()*16 + _getbkcolor();
	rc = _gettextposition();
	t->curx = rc.col;
	t->cury = rc.row;
	_gettextwindow( &top, &left, &bottom, &right );
	t->wintop = top;
	t->winleft = left;
	t->winright = right;
	t->winbottom = bottom;
}

void clreol()
{
}

void textattr(int attr)
{
	int bg, fg;

	fg = attr / 16;
	bg = attr % 16;
	_settextcolor(fg);
	_setbkcolor(bg);
}
#endif

