/*=========================================================================*/
/* XLLIST.H contains the data structure definitions for the virtual memory */
/*          access functions and the functions' prototypes and macro       */
/*          definitions as well.                                           */
/*=========================================================================*/


/************************************************************************/
/*									*/
/*						 XL DATASOURCE, Inc.	*/
/*  Copyright (C) 1988, 1989, 1990	      PO Box 28-B   Station 16	*/
/*						 Lakewood, CO  80228	*/
/*									*/
/************************************************************************/

#if defined(MAIN) && !defined(GLOBALDEF)
  #define INIT(x) x
  #define GLOBALDEF
#elif !defined(MAIN) && !defined(GLOBALDEF)
  #define INIT(x)
  #define GLOBALDEF extern
#endif

#ifndef BYTE
#define BYTE unsigned char
#endif
#ifndef BOOLEAN
#define BOOLEAN unsigned char
#endif

#define TRUE  1
#define FALSE 0
#define YES 1
#define NO 0

GLOBALDEF  long HARD_DISK_HEADROOM  INIT(=50000L);
GLOBALDEF  long RAM_HEADROOM  INIT(=90000L);

GLOBALDEF char   *VMDISK;                /* pointer to 'XLLIST.C' Disk path      */
GLOBALDEF char   crit_error;             /* critical error flag */

#define ever ;;
#define ASCENDING_SEQ     0x10     /* collating sequence directions... */
#define DESCENDING_SEQ    0x20


#include <stdio.h>
#include "bitmap.h"
#include <dos.h>
#include <stdlib.h>
#include <stdarg.h>
#include <ctype.h>
#include <math.h>
#include <mem.h>
#include <string.h>

/***** INDEX.ASM ***/
int xl_index(char *s, char c);
#define index(s,c)	xl_index(s,c)

/* from CEFILE.C */
/* prototypes for critical error controlled stream functions */
char  *ce_fgets(char *s, int n, FILE *stream);
int    ce_fprintf(FILE *stream, const char *format, ...);
int    ce_access(const char *filename, int amode);
FILE  *ce_fopen(const char *filename, const char *mode);
int    ce_fclose(FILE *stream);
size_t ce_fwrite(const void *ptr, size_t size, size_t n, FILE *stream);
size_t ce_fread(void *ptr, size_t size, size_t n, FILE *stream);
int    ce_fseek(FILE *stream, long offset, int whence);
long   ce_ftell(FILE *stream);


/* the order of the media "defines" below can vary according to desired
   precedence of use (and THAT generally depends upon which is fastest,
   although one could make a case for a different ordering...)
   in general the speed of access of the 4 media is (fastest to slowest)
   RAM, EMS, XMS, DISK --- HOWEVER some machines use EMS emulators which
   actually use extended memory for storage -- this extra layer of software
   can make (fake) EMS run slower than direct XMS access -- the only good
   thing about it (besides enabling some use of XMS for programs that use
   EMS only) is the standard means of access */

/* tell makeLIST to use the default medium */
#define VM_DEFAULT 0

#define IN_EMS  1
#define IN_XMS  2
#define IN_RAM  3
#define ON_DISK 4

/* sum of all 4 of the above media */
#define VM_TOTAL_SUM 10

#define NUM_VIRT_MEDIA 4
GLOBALDEF BYTE media_list[NUM_VIRT_MEDIA + 1]
#ifdef MAIN
= {IN_EMS, IN_XMS, IN_RAM, ON_DISK, 0}
#endif
;

/* virtual memory access structures */
/* generic xms information for virtual memory system */
typedef struct xm_info
{
    unsigned total_kbytes;
    unsigned available_kbytes;
} XMS_INFO;

/* generic ems information for virtual memory system */
typedef struct em_info
{
    void *ems_base_addr;   /* seg:0000 of EMS page frame */
    int current_handle;    /* currently mapped pages belong to this handle */
    unsigned total_pages;
    unsigned available_pages;
} EMS_INFO;

/*===== INDEX STRUCTURES =====*/

typedef struct ems_data_addr
{
    int      handle;
    BYTE     num_pages;
    unsigned offset;
} EMS_DATA_ADDR;

/* data index lookup info -- if you change the size of this structure
   you must also change DINDEX_SIZE as defined in XLLIST.C */
typedef struct data_index
{
    union            /* address of the data */
    {
	void          *ram_addr;
	unsigned long xm_addr;
	EMS_DATA_ADDR em_addr;
	long          dsk_addr;
    } addr;

    BYTE           medium;               /* which storage medium */
    unsigned       data_len;              /* data length */
} DINDEX;

/*===== INDEX ACCESS STRUCTURES =====*/

typedef struct ria
{
    long       first_index, last_index;
    DINDEX     *ram_addr;               /* base address */
    struct ria *chain;                  /* pointer to next index table */
} RAM_INDEX_ACCESS;

typedef struct eia
{
    long       first_index, last_index;
    int        handle;
    unsigned   num_pages;
    DINDEX     *em_addr;
    struct eia *chain;
} EMS_INDEX_ACCESS;

typedef struct xia
{
    long          first_index, last_index;
    int           handle;
    unsigned      allocated_kbytes;        /* bytes to deallocate */
    unsigned long xm_addr;
    struct xia    *chain;
} XMS_INDEX_ACCESS;

typedef struct dia
{
    long          first_index, last_index;
    FILE          *vmfp;
    char          fname[80];
    struct dia    *chain;
    int		  fileno;
} DSK_INDEX_ACCESS;


/*===== DATA ACCESS STRUCTURES =====*/

typedef struct rda
{
    char       *ram_base;
    char       *ram_addr;
    struct rda *chain;
} RAM_DATA_ACCESS;

typedef struct eda
{
    int           handle;
    unsigned      num_pages;
    void          *em_addr;
    struct eda    *chain;
} EMS_DATA_ACCESS;

typedef struct xda
{
    int           handle;
    unsigned      allocated_kbytes;
    unsigned long xm_addr;
    struct xda    *chain;
} XMS_DATA_ACCESS;

typedef struct dda
{
    FILE          *vmfp;
    char          fname[80];
    unsigned long dsk_addr;
    struct dda    *chain;
} DSK_DATA_ACCESS;

typedef struct cia
{
    long       first_index, last_index;
    DINDEX     *ram_addr;               /* base address */
    unsigned   num_bytes;               /* cache size */
} CACHE_INDEX_ACCESS;

/*===== LIST CONTROL BLOCK =====*/

#define INDEX_TABLE_SIZE 45

typedef struct list_control_block
{
    long             next_index;         /* 0... 2 billion+ */
    long             user_counter;	 /* our current location */
    BYTE             list_error;         /* error codes for list routines */
    BYTE             build_index_medium; /* the four build variables */
    BYTE             build_data_medium;  /* contain information about the */
    unsigned long    build_dbytes_left;  /* current state of the list for */
    unsigned         build_islots_left;  /* purposes of fast insertion */
    char             *lbuffer;           /* temp data return buffer */
    DINDEX           dindex;             /* internal work space index record */
    BITMAP           bitmap;             /* 1 bit per list element -- for */
					 /* marking a sublist */
    RAM_INDEX_ACCESS *ram_index;
    RAM_DATA_ACCESS  *ram_data;
    EMS_INDEX_ACCESS *ems_index;
    EMS_DATA_ACCESS  *ems_data;
    XMS_INDEX_ACCESS *xms_index;
    XMS_DATA_ACCESS  *xms_data;
    DSK_INDEX_ACCESS *dsk_index;
    DSK_DATA_ACCESS  *dsk_data;
    CACHE_INDEX_ACCESS *cache;
    long index_table[INDEX_TABLE_SIZE];
} LCB;

/*=========================================================================*/

/*===== PROTOTYPES AND MACROS FOR VIRTUAL LIST ACCESS FUNCTIONS =====*/

/* verify existence of and initialize any virtual memory media present --
   this function MUST be called before using the virtual memory access
   functions below or only RAM memory will be used */
void init_vmm(void);

/* shut down virtual memory system -- free all LCB structures still
   in existence */
void close_vmm(void);

/* create and add items to a virtual memory list -- returns the number
   of the list element (its handle) just added OR -1 if an error occured */
long xl_makeLIST(LCB **lcb, BYTE medium, unsigned data_len, void *data);
#define makeLIST(l,m,dl,d) xl_makeLIST(&l,m,dl,d)

/* return a pointer to the data indexed by a specified handle -- the pointer
   is good until the next call to getLIST at which time it will be destroyed
   (calls to sortLIST, which calls getLIST internally, also modify this
   pointer) -- if the returned data must remain intact through several
   calls to getLIST, it must be copied to a temporary buffer for that time.
   -- returns a NULL pointer if the item does not exists or if an error
   occured while attempting to access the information */
void *getLIST(LCB *lcb, long element_number);

/* update the data stored at a specified handle -- the length of the new
   information must be <= the length of the buffer as it was created --
   returns TRUE if all went well, FALSE if there was an error */
BOOLEAN updateLIST(LCB *lcb, long element_number, void *in_data,
		   unsigned data_len);

/* save a virtual list to disk using filename fname (a filename
   without an extension) -- the index and data portions of the
   list are retained on disk as fname.vmx and fname.vmd and may
   be reloaded and used at any time -- currently the saved list
   MUST BE ONE WHICH HAS BEEN WRITTEN ENTIRELY TO DISK IN THE
   FIRST PLACE i.e. makeLIST was called throughout with a parameter
   of ON_DISK as medium (if there was not enough disk space to
   contain the entire list the portion not on disk won't be saved
   we'll redo it later to work more generically
*/
#define saveLIST(l,f) xl_zapLIST(&l,f)

/* create an lcb structure to manage a saved list -- open list
   files etc. -- return a pointer to the lists lcb -- currently
   a restored list will be managed as a disk list ONLY -- later
   we may load it into chip memory for performance
*/
LCB *restoreLIST(char *fname, char *listmode);

#define LIST_READ_ONLY  "rb"
#define LIST_READ_WRITE "r+b"

/* binary search for a text match in a SORTED virtual list/array -- finds
   "text" at offset "sofs" for a length of "slen" bytes on a list/array which
   has been sorted in a "sort_mode" manner (i.e. ASCENDING_SORT or
   DESCENDING_SORT) -- low and high are the indices between which to search
   RETURNS -1 if not found, or the index number of the element if found
   (lcb->lbuffer contains the found data) */
long bsearchLIST (LCB *lcb, void *text, long low, long high,
		 unsigned sofs, unsigned slen,
		 BYTE sort_mode);

/* linear search for text match in any virtual list/array -- similar to the
   bsearch above except that the search direction is determined in the
   following manner -- if low < high the search proceeds from low to high with
   low increasing (a forward search) -- if low > high the search proceeds for
   low to high with low decreasing ( a backward search) -- return values are
   identical to the above bsearch */
long lsearchLIST (LCB *lcb, void *text, long low, long high,
		 unsigned sofs, unsigned slen);

/* sorts the elements in a virtual list/array -- user supplies the
   comparison function -- if NULL is passed as the comparison function
   pointer a default function is used -- uses a quicksort (with an auxillary
   insertion sort when the quicksort partitions become small enough) */
void sortLIST(LCB *lcb, int (*cmp)());

/* destroy the entire virtual memory structure and set the lcb to NULL */
#define zapLIST(l) xl_zapLIST(&l, NULL)
void xl_zapLIST(LCB **lcb, char *fname);

/*=========================================================================*/